function [H, opt] = assemble(G, opt)
%% stiffness and mass matrix assembly, as full matrix or as low rank TensorTrain with NURBS or B-splines, in 2D or 3D
% input: geometry structure G as assembled by readGeo and strcuture opt 
% with the following [1/0] options [default]
% plotW:        plots grid of weight approximation accuracy [0]
% lowRank:      reduce rank for the SVD [1]
% mass:         1 if mass matrix should be assembled [1]
% stiffness:    1 if stiffness matrix should be assembled [1]
% discardFull:  discards the assembled weight matrices [0]
% TTT:          1 if tensor train structure, 0 if complete matrices [0]
% and following options, which require pre-estimated values if chosen,
% else leave blank:
% sizeLowRank:  fixed size for the SVD appriximation as integer, []
% rankTol:      tolerance for rank truncation [default 1e-5]

%% The system matrices will be assembled by the following steps (ex. in 2D)
% 1. an approximation of the weight function W(s,t) = S(s)W T(t)
% 2. a low rank approximation of weights W = U sigma V
% 3. univariate integration
% 4. combining the univariate parts via kroenecker product or storing them
% seperately for usage in TT-Toolbox or similar

% detailed information on each step is given in the according file

%% interpolation of the weight matrix with higher dimensional Spline Space
[H, opt] = interpolateWeightsNURBS(G, opt);


%% low rank SVD
[H, opt] = lowRankSVD(H, opt);

%% univariate integrals of S and T for matrices
if G.dim == 2
    [H, opt] = univariateIntegralSGauss(H, opt);
    [H, opt] = univariateIntegralTGauss(H, opt);
elseif G.dim == 3
    [H, opt] = univariateIntegralXGauss(H, opt);
    [H, opt] = univariateIntegralYGauss(H, opt);
    [H, opt] = univariateIntegralZGauss(H, opt);
end

% fix boundary conditions
if isfield(opt, 'nbd') && opt.nbd == 1 % no other boundary conditions than 0 Dirichlet implemented yet
     %% ---
else
    H.boundary.diriclet = 1;
     if G.dim == 2
         boundaryS = evalNURBS(H.geometry.tensor.knots{1}, H.geometry.tensor.degree(1), H.geometry.tensor.Tweights{1}', [0,1]);
         boundaryT = evalNURBS(H.geometry.tensor.knots{2}, H.geometry.tensor.degree(2), H.geometry.tensor.Tweights{2}', [0,1]);
         H.sInner = real((boundaryS(:,1) == 0) & (boundaryS(:,2) == 0));
         H.tInner = real((boundaryT(:,1) == 0) & (boundaryT(:,2) == 0));
     elseif G.dim == 3
         boundaryX = evalNURBS(H.geometry.tensor.knots{1}, H.geometry.tensor.degree(1), H.geometry.tensor.Tweights{1}', [0,1]);
         boundaryY = evalNURBS(H.geometry.tensor.knots{2}, H.geometry.tensor.degree(2), H.geometry.tensor.Tweights{2}', [0,1]);
         boundaryZ = evalNURBS(H.geometry.tensor.knots{3}, H.geometry.tensor.degree(3), H.geometry.tensor.Tweights{3}', [0,1]);
         H.boundary.xInner = real((boundaryX(:,1) == 0) & (boundaryX(:,2) == 0));
         H.boundary.yInner = real((boundaryY(:,1) == 0) & (boundaryY(:,2) == 0));
         H.boundary.zInner = real((boundaryZ(:,1) == 0) & (boundaryZ(:,2) == 0));
     end
end

%% Assemble matrices in right format
[H, opt] = matrixFormat(H, opt);

end