function [H, opt] = interpolateWeightsNURBS(G, opt)

% dimS = number of basis functions for spline space S, dimS = numel(knotsS)-degreeS-1
% dimT = number of basis functions for spline space T, dimT = numel(knotsT)-degreeT-1
% size(controlPoints) == [dimS, dimT, 2]
%
% We will compute two spline spaces S2 and T2 and corresponding weights to
% interpolate the four entries of the 2x2 matrix function 
%       Q(s,t) = abs(det(Jac G(s,t))) * inv(Jac G(s,t)) * inv(Jac G(s,t))',
% where G is the 2D spline product function specified by the control
% points.
% Afterwards, the 3D weights matrix holds the coefficients such that
%    Q_11(s,t) = evalBSpline(knotsS2, degreeS2, s)' * weights(:,:,1) * evalBSpline(knotsT2, degreeT2, t)
%    Q_12(s,t) = evalBSpline(knotsS2, degreeS2, s)' * weights(:,:,2) * evalBSpline(knotsT2, degreeT2, t)
%    Q_22(s,t) = evalBSpline(knotsS2, degreeS2, s)' * weights(:,:,3) * evalBSpline(knotsT2, degreeT2, t)
% and Q_21 == Q_12 because Q is symmetric.
%
% for k,l = 1,2.
%
% For s, t fixed, let valS = evalBSpline(knotsS, degreeS, s),
%                     valT = evalBSpline(knotsT, degreeT, t),
%                     derS = evalBSplineDeriv(knotsS, degreeS, s),
%                     derT = evalBSplineDeriv(knotsT, degreeT, t).
% Then G(s,t) = [ valS' * controlPoints(:,:,1) * valT; 
%                 valS' * controlPoints(:,:,2) * valT ]
%        = \sum_i \sum_j valS(i)*controlPoints(i,j,:)*valT(j).
%
% Its Jacobi matrix is given by Jac G(s, t) = \sum_i \sum_j
%                   [ derS(i)*controlPoints(i,j,:)*valT(j), valS(i)*controlPoints(i,j,:)*derT(j) ]
%         = [ derS'*controlPoints(:,:,1)*valT, valS'*controlPoints(:,:,1)*derT;
%             derS'*controlPoints(:,:,2)*valT, valS'*controlPoints(:,:,2)*derT ].
% 
% Seeing that Jac G(s,t) = 1/det(Jac G(s,t)) * H(s,t) with
%     H(s, t) = [  valS'*controlPoints(:,:,2)*derT, - valS'*controlPoints(:,:,1)*derT
% Q(s,t) = 1/abs(Jac G(s,t))*[j22^2+j12^2, -j22*j21-j12*j11;
% -j22*j21-j12*j11, j21^2+j11^2];
%

if nargin < 2
    opt = struct();
end
if ~isfield(opt, 'plotW') || isempty(opt.plotW)
    opt.plotW = 0;
end
if ~isfield(opt, 'stiffness') || isempty(opt.stiffness)
    opt.stiffness = 0;
end
if ~isfield(opt, 'mass') || isempty(opt.mass)
    opt.mass = 0;
end
if ~isfield(opt,'greville')
    opt.greville = 1;
end
if opt.mass == 0 && opt.stiffness == 0
    error('Please specify a system matrix to be computed');
end
if ~isfield(G, 'tensor') && G.nurbs == 0
    G.tensor = G;
    G.tensor.Tweights = tt_tensor(ones(G.n));
elseif ~isfield(G,'tensor') && G.nurbs == 1
    error('Geometry not given in tensor structure');
end

if ~isfield(opt,'splinespace2') || isempty(opt.splinespace2)
    opt.splinespace2 = 0;
end

if ~isfield(opt,'splinedegree2') || isempty(opt.splinedegree2)
    opt.splinedegree2 = 0;
end

H = struct;
H.geometry = G;
G.weightFun = struct;

%% Create larger spline spaces for interpolation
H.weightFun.knots = cell(G.dim,1);
H.weightFun.degree = zeros(G.dim,1);
H.weightFun.weight = cell(G.dim,1);
H.weightFun.n = zeros(G.dim,1);
for i = 1:G.dim
    [knots2, degree2] = enlargenNURBSSpace(G.knots{i}, G.degree(i),opt.splinespace2, opt.splinedegree2);
    H.weightFun.knots{i} = knots2;
    H.weightFun.degree(i) = degree2;
    H.weightFun.n(i) = numel(knots2) - degree2 -1;
    H.weightFun.weight{i} = ones(1,H.weightFun.n(i));
end

grevillePoints = cell(H.geometry.dim,1);
grevilleValues = cell(H.geometry.dim,1);
grevilleDerivs = cell(H.geometry.dim,1);
grevilleValues2 = cell(H.geometry.dim,1);
%% Generate Greville points and evaluate basis splines there
for i = 1:H.geometry.dim
    grevillePoints{i} = opt.greville*generateGrevillePoints(H.weightFun.knots{i}, H.weightFun.degree(i));     
    % Compute basis spline values on greville points
    grevilleValues{i} =  sparse(evalNURBS      (G.tensor.knots{i}, G.tensor.degree(i), G.tensor.Tweights{i}', grevillePoints{i}));
    grevilleDerivs{i} =  sparse(evalNURBSDeriv (G.tensor.knots{i}, G.tensor.degree(i), G.tensor.Tweights{i}', grevillePoints{i}));
    grevilleValues2{i} = sparse(evalNURBS      (H.weightFun.knots{i}, H.weightFun.degree(i), H.weightFun.weight{i}', grevillePoints{i}));
end

%% Setup equation matrix and right hand sides

if H.geometry.dim == 2
    % Each row in grevillePoints2D contains [s t] coordinates of a
    % combination of a point in grevillePointsS and one in grevillePointsT.
    % The number of rows is dimS2*dimT2.
    M = kron(grevilleValues2{2}',grevilleValues2{1}');
    jac1 = grevilleDerivs{1}' * G.tensor.controlPoints(:,:,1) * grevilleValues{2};
    jac2 = grevilleDerivs{1}' * G.tensor.controlPoints(:,:,2) * grevilleValues{2};
    jac3 = grevilleValues{1}' * G.tensor.controlPoints(:,:,1) * grevilleDerivs{2};
    jac4 = grevilleValues{1}' * G.tensor.controlPoints(:,:,2) * grevilleDerivs{2};
    w = abs(jac1.*jac4-jac3.*jac2);
    if opt.stiffness == 1
        q1 = reshape(1./w .* (jac4.^2 + jac3.^2),prod(H.weightFun.n),1);
        q2 = reshape(-1./w .*(jac2.*jac4 + jac1.*jac3), prod(H.weightFun.n),1);
        q3 = reshape(1./w .*(jac1.^2 + jac2.^2), prod(H.weightFun.n),1);
        vecWeights = M \ [q1,q2,q3];
        H.stiffness.weightMat = reshape(vecWeights, H.weightFun.n(1), H.weightFun.n(2), 3);
    end
    if opt.mass == 1
        w1 = reshape(w, H.weightFun.n(1)*H.weightFun.n(2),1);
        H.mass.weightMat = reshape(M\w1, H.weightFun.n(1),H.weightFun.n(2));
    end
elseif H.geometry.dim == 3
    A = tt_matrix({grevilleDerivs{1}'; grevilleValues{2}'; grevilleValues{3}'});
    jac11 = A*G.tensor.controlPoints(:,1);
    jac12 = A*G.tensor.controlPoints(:,2);
    jac13 = A*G.tensor.controlPoints(:,3);
    A = tt_matrix({grevilleValues{1}'; grevilleDerivs{2}'; grevilleValues{3}'});
    jac21 = A*G.tensor.controlPoints(:,1);
    jac22 = A*G.tensor.controlPoints(:,2);
    jac23 = A*G.tensor.controlPoints(:,3);
    A = tt_matrix({grevilleValues{1}'; grevilleValues{2}'; grevilleDerivs{3}'});
    jac31 = A*G.tensor.controlPoints(:,1);
    jac32 = A*G.tensor.controlPoints(:,2);
    jac33 = A*G.tensor.controlPoints(:,3);
    w = abs(jac11.*jac22.*jac33 + jac12.*jac23.*jac31 + jac21.*jac32.*jac13 - jac13.*jac22.*jac31 - jac12.*jac21.*jac33 - jac11.*jac23.*jac32);
    nswp = 10;
    if opt.stiffness == 1
        m11 = jac22.*jac33 - jac32.*jac23;
        m12 = jac12.*jac33 - jac32.*jac13;
        m13 = jac12.*jac23 - jac22.*jac13;
        m21 = jac21.*jac33 - jac31.*jac23;
        m22 = jac11.*jac33 - jac31.*jac13;
        m23 = jac11.*jac23 - jac21.*jac13;
        m31 = jac21.*jac32 - jac31.*jac22;
        m32 = jac11.*jac32 - jac31.*jac12;
        m33 = jac11.*jac22 - jac21.*jac12;
        q11 = 1./w.*(m11.^2 + m21.^2 + m31.^2);
        q12 = 1./w.*(-m11.*m12 - m21.*m22 - m31.*m32);
        q13 = 1./w.*(m11.*m13 + m21.*m23 + m31.*m33);
        q22 = 1./w.*(m12.^2 + m22.^2 + m32.^2);
        q23 = 1./w.*(-m12.*m13 - m22.*m23 - m32.*m33);
        q33 = 1./w.*(m13.^2 + m23.^2 + m33.^2);
        
        if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 0
            M= kron(grevilleValues2{3},kron(grevilleValues2{2},grevilleValues2{1}))';
            H.stiffness.weightMat = cell(6,1);
        else
            MM = {grevilleValues2{1}';grevilleValues2{2}';grevilleValues2{3}'};
            H.stiffness.weightMat = cell(6,1);
            opt.TT_interpolation = 1;
        end
        if sum(abs(q11))/numel(q11) > 1e-14
            if opt.TT_interpolation == 1
                tt_11 = tt_tensor(reshape(q11, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{1} = amen_block_solve({MM},{tt_11},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{1} = reshape(M\q11, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end
        if sum(abs(q12))/numel(q12) > 1e-14
            if opt.TT_interpolation == 1
                tt_12 = tt_tensor(reshape(q12, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{2} = amen_block_solve({MM},{tt_12},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{2} = reshape(M\q12, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end
        if sum(abs(q13))/numel(q13) > 1e-13
            if opt.TT_interpolation == 1
                tt_13 = tt_tensor(reshape(q13, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{3} = amen_block_solve({MM},{tt_13},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{3} = reshape(M\q13, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end
        if sum(abs(q22))/numel(q22) > 1e-14
            if opt.TT_interpolation == 1
                tt_22 = tt_tensor(reshape(q22, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{4} = amen_block_solve({MM},{tt_22},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{4} = reshape(M\q22, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end        
        if sum(abs(q23))/numel(q23) > 1e-14
            if opt.TT_interpolation == 1
                tt_23 = tt_tensor(reshape(q23, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{5} = amen_block_solve({MM},{tt_23},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{5} = reshape(M\q23, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end     
        if sum(abs(q33))/numel(q33) > 1e-14
            if opt.TT_interpolation == 1
                tt_33 = tt_tensor(reshape(q33, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
                H.stiffness.weightMat{6} = amen_block_solve({MM},{tt_33},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
            else
                H.stiffness.weightMat{6} = reshape(M\q33, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
            end
        end

    end
    if opt.mass == 1
        if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
            MM = {grevilleValues2{1}';grevilleValues2{2}';grevilleValues2{3}'};
            tt_w = tt_tensor(reshape(w, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
             H.mass.weightMat= amen_block_solve({MM},{tt_w},opt.rankTol, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', nswp, 'exitdir', -1);
        else
            M= kron(grevilleValues2{3},kron(grevilleValues2{2},grevilleValues2{1}))';
            vecWeights = M\w; 
            H.mass.weightMat = reshape(vecWeights, H.weightFun.n(1), H.weightFun.n(2), H.weightFun.n(3));
        end
    end
end


%% Plot error if desired

if opt.plotW == 1
     if H.geometry.dim == 2
        jacobi = zeros(2);
        h = 50;
        ss = linspace(G.tensor.knots{1}(1), G.tensor.knots{1}(end), h);
        tt = linspace(G.tensor.knots{2}(1), G.tensor.knots{2}(end), h);
        [SS, TT] = meshgrid(ss, tt);
        XX = zeros(size(SS));
        YY = zeros(size(SS));
        ZZ = zeros([size(SS),4]);
        ZZWeights = zeros([size(SS),4]);
        for i = 1:size(SS,1)
            for j = 1:size(SS,2)
                valS  = evalNURBS     (G.tensor.knots{1},  G.tensor.degree(1), G.tensor.Tweights{1}', SS(i,j));
                derS  = evalNURBSDeriv(G.tensor.knots{1},  G.tensor.degree(1), G.tensor.Tweights{1}', SS(i,j));
                valS2 = evalNURBS     (H.weightFun.knots{1}, H.weightFun.degree(1), H.weightFun.weight{1}', SS(i,j));
                valT  = evalNURBS     (G.tensor.knots{2},  G.tensor.degree(2), G.tensor.Tweights{2}', TT(i,j));
                derT  = evalNURBSDeriv(G.tensor.knots{2},  G.tensor.degree(2), G.tensor.Tweights{2}', TT(i,j));
                valT2 = evalNURBS     (H.weightFun.knots{2}, H.weightFun.degree(2), H.weightFun.weight{2}', TT(i,j));

                XX(i,j) = valS' * G.tensor.controlPoints(:,:,1) * valT;
                YY(i,j) = valS' * G.tensor.controlPoints(:,:,2) * valT;

                jacobi(1, 1) = derS' * G.tensor.controlPoints(:,:,1) * valT;
                jacobi(2, 1) = derS' * G.tensor.controlPoints(:,:,2) * valT;
                jacobi(1, 2) = valS' * G.tensor.controlPoints(:,:,1) * derT;
                jacobi(2, 2) = valS' * G.tensor.controlPoints(:,:,2) * derT;

                absDet = abs(det(jacobi));         
                if opt.stiffness == 1
                    ZZ(i,j,1) =  1/absDet * (jacobi(2, 2)*jacobi(2, 2) + jacobi(1, 2)*jacobi(1, 2));
                    ZZ(i,j,2) = -1/absDet * (jacobi(2, 1)*jacobi(2, 2) + jacobi(1, 1)*jacobi(1, 2));
                    ZZ(i,j,3) =  1/absDet * (jacobi(1, 1)*jacobi(1, 1) + jacobi(2, 1)*jacobi(2, 1));
                    ZZWeights(i,j,1) = valS2' * H.stiffness.weightMat(:,:,1) * valT2;
                    ZZWeights(i,j,2) = valS2' * H.stiffness.weightMat(:,:,2) * valT2;
                    ZZWeights(i,j,3) = valS2' * H.stiffness.weightMat(:,:,3) * valT2;
                end
                if opt.mass == 1
                    ZZ(i,j,4) = abs(det(jacobi));
                    ZZWeights(i,j,4) = valS2' * H.mass.weightMat * valT2;
                end
                if absDet <= 0
                    fprintf('Determinant is zero on Plot Point (%d, %d)\n', SS(i,j), TT(i,j));
                end
             end
        end
        figure;
         if opt.stiffness == 1
             subplot(2,2,1);
             surf(XX, YY, log(abs(ZZ(:,:,1)-ZZWeights(:,:,1)))/log(10))
             colorbar;
             title('Error for first entry of Stffness Weight Matrix Interpolation')
             zlabel('Log Scale of Absolute Error')
             subplot(2,2,2);
             surf(XX, YY, log(abs(ZZ(:,:,2)-ZZWeights(:,:,2)))/log(10))
             colorbar;
             title('Error for second entry of Stiffness Weight Matrix Interpolation')
             zlabel('Log Scale of Absolute Error')
             subplot(2,2,3);
             surf(XX, YY, log(abs(ZZ(:,:,3)-ZZWeights(:,:,3)))/log(10))
             colorbar;
             title('Error for third entry of Stiffness Weight Matrix Interpolation')
             zlabel('Log Scale of Absolute Error')
             subplot(2,2,4);
             surf(XX,YY,ZZ(:,:,1))
             colorbar;
         end
         if opt.mass == 1
             surf(XX, YY, log(abs(ZZ(:,:,4) - ZZWeights(:,:,4))/log(10)))
             title('Error for Mass Weights Interpolation')
             zlabel('Log Scale of Absolute Error')
         end
        fprintf('relative error for entry 1 %d \n',sum(sum(abs(ZZ(:,:,1)-ZZWeights(:,:,1))./abs(ZZ(:,:,1))))/(h^2));
        fprintf('relative error for entry 2 %d \n',sum(sum(abs(ZZ(:,:,2)-ZZWeights(:,:,2))./abs(ZZ(:,:,1))))/(h^2));
        fprintf('relative error for entry 3 %d \n',sum(sum(abs(ZZ(:,:,3)-ZZWeights(:,:,3))./abs(ZZ(:,:,1))))/(h^2));
     elseif H.geometry.dim == 3
         h = 10;
         xx = cell(H.geometry.dim,1);
         values = cell(H.geometry.dim,1);
         derivs = cell(H.geometry.dim,1);
         values2 = cell(H.geometry.dim,1);
         for i = 1:3
             xx{i} = linspace(G.tensor.knots{i}(1), G.tensor.knots{i}(end), h);
             values{i} = sparse(evalNURBS(G.tensor.knots{i}, G.tensor.degree(i), G.tensor.Tweights{i}', xx{i}));
             derivs{i} = sparse(evalNURBSDeriv(G.tensor.knots{i}, G.tensor.degree(i), G.tensor.Tweights{i}', xx{i}));
             values2{i} = sparse(evalNURBS(H.weightFun.knots{i}, H.weightFun.degree(i), H.weightFun.weight{i}',xx{i}));
         end
         jac11 = kron(values{3}, kron(values{2}, derivs{1}))'*G.tensor.controlPoints(:,1);
         jac21 = kron(values{3}, kron(derivs{2}, values{1}))'*G.tensor.controlPoints(:,1);
         jac31 = kron(derivs{3}, kron(values{2}, values{1}))'*G.tensor.controlPoints(:,1);
         jac12 = kron(values{3}, kron(values{2}, derivs{1}))'*G.tensor.controlPoints(:,2);
         jac22 = kron(values{3}, kron(derivs{2}, values{1}))'*G.tensor.controlPoints(:,2);
         jac32 = kron(derivs{3}, kron(values{2}, values{1}))'*G.tensor.controlPoints(:,2);
         jac13 = kron(values{3}, kron(values{2}, derivs{1}))'*G.tensor.controlPoints(:,3);
         jac23 = kron(values{3}, kron(derivs{2}, values{1}))'*G.tensor.controlPoints(:,3);
         jac33 = kron(derivs{3}, kron(values{2}, values{1}))'*G.tensor.controlPoints(:,3);
         w = abs(jac11.*jac22.*jac33 + jac12.*jac23.*jac31 + jac21.*jac32.*jac13 - jac13.*jac22.*jac31 - jac12.*jac21.*jac33 - jac11.*jac23.*jac32);
         
         eps = 1e-5;
         if opt.mass == 1
             if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
                 absDetWeights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.mass.weightMat);
             else
                absDetWeights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.mass.weightMat, prod(H.weightFun.n),1);
             end
            fprintf('Mean relative error for mass matrix interpolation is %d . \n', sum(abs((w-absDetWeights)./(eps+w)))/(h^3));
            figure
            plot(w,'*');
            hold on
            plot(absDetWeights,'ro');
         end
         if opt.stiffness == 1
            m11 = jac22.*jac33 - jac32.*jac23;
            m12 = jac12.*jac33 - jac32.*jac13;
            m13 = jac12.*jac23 - jac22.*jac13;
            m21 = jac21.*jac33 - jac31.*jac23;
            m22 = jac11.*jac33 - jac31.*jac13;
            m23 = jac11.*jac23 - jac21.*jac13;
            m31 = jac21.*jac32 - jac31.*jac22;
            m32 = jac11.*jac32 - jac31.*jac12;
            m33 = jac11.*jac22 - jac21.*jac12;
            q11 = 1./w.*(m11.^2 + m21.^2 + m31.^2);
            q12 = 1./w.*(-m11.*m12 - m21.*m22 - m31.*m32);
            q13 = 1./w.*(m11.*m13 + m21.*m23 + m31.*m33);
            q22 = 1./w.*(m12.^2 + m22.^2 + m32.^2);
            q23 = 1./w.*(-m12.*m13 - m22.*m23 - m32.*m33);
            q33 = 1./w.*(m13.^2 + m23.^2 + m33.^2);
            if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
                figure;
                if ~isempty(H.stiffness.weightMat{1})
                    q11Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{1});
                    subplot(3,2,1)
                    plot(q11,'*');
                    hold on;
                    plot(q11Weights,'ro');
                    title('Q_{1,1}')
                    fprintf('Mean error for stiffness matrix interpolation Q_11 is %d, \n', sum((q11-q11Weights).^2)/sum(q11.^2))
                end
                if ~isempty(H.stiffness.weightMat{2})
                    q12Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{2});
                    subplot(3,2,2);
                    plot(q12,'*');
                    hold on;
                    plot(q12Weights, 'ro');
                    title('Q_{1,2}')
                    fprintf('Mean error for stiffness matrix interpolation Q_12 is %d, \n', sum((q12-q12Weights).^2)/sum(q12.^2))
                end
                if ~isempty(H.stiffness.weightMat{3})
                    q13Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{3});
                    subplot(3,2,3);
                    plot(q13,'*');
                    hold on;
                    plot(q13Weights, 'ro');
                    title('Q_{1,3}')
                    fprintf('Mean absolut error for stiffness matrix interpolation Q_13 is %d, \n', sum((q13-q13Weights).^2)/sum(q13.^2))
                end
                if ~isempty(H.stiffness.weightMat{4})
                    q22Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{4});
                    subplot(3,2,4);
                    plot(q22,'*');
                    hold on;
                    plot(q22Weights, 'ro');
                    title('Q_{2,2}')
                    fprintf('Mean absolut error for stiffness matrix interpolation Q_22 is %d, \n', sum((q22-q22Weights).^2)/sum(q22.^2))
                end
                if ~isempty(H.stiffness.weightMat{5})
                    q23Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{5});
                    subplot(3,2,5);
                    plot(q23,'*');
                    hold on;
                    plot(q23Weights, 'ro');
                    title('Q_{2,3}')
                    fprintf('Mean absolut error for stiffness matrix interpolation Q_23 is %d, \n', sum((q23-q23Weights).^2)/sum(q23.^2))
                end
                if ~isempty(H.stiffness.weightMat{6})
                    q33Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*full(H.stiffness.weightMat{6});
                    subplot(3,2,6);
                    plot(q33,'*');
                    hold on;
                    plot(q33Weights, 'ro');
                    title('Q_{3,3}')
                    fprintf('Mean absolut error for stiffness matrix interpolation Q_33 is %d, \n', sum((q33-q33Weights).^2)/sum(q33.^2))
                end
            else
                q11Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,1), prod(H.weightFun.n),1);
                q12Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,2), prod(H.weightFun.n),1);
                q13Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,3), prod(H.weightFun.n),1);
                q22Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,4), prod(H.weightFun.n),1);
                q23Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,5), prod(H.weightFun.n),1);
                q33Weights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(H.stiffness.weightMat(:,:,:,6), prod(H.weightFun.n),1);
                figure;
                subplot(3,2,1)
                plot(q11,'*');
                hold on;
                plot(q11Weights,'ro');
                title('Q_{1,1}')
                subplot(3,2,2);
                plot(q12,'*');
                hold on;
                plot(q12Weights, 'ro');
                title('Q_{1,2}')
                subplot(3,2,3);
                plot(q13,'*');
                hold on;
                plot(q13Weights, 'ro');
                title('Q_{1,3}')
                subplot(3,2,4);
                plot(q22,'*');
                hold on;
                plot(q22Weights, 'ro');
                title('Q_{2,2}')
                subplot(3,2,5);
                plot(q23,'*');
                hold on;
                plot(q23Weights, 'ro');
                title('Q_{2,3}')
                subplot(3,2,6);
                plot(q33,'*');
                hold on;
                plot(q33Weights, 'ro');
                title('Q_{3,3}')
                fprintf('Mean absolut error for stiffness matrix interpolation Q_11 is %d, \n', sum(abs((q11-q11Weights)./(eps+q11)))/h^3)
                fprintf('Mean absolut error for stiffness matrix interpolation Q_12 is %d, \n', sum(abs((q12-q12Weights)./(eps+q12)))/h^3)
                fprintf('Mean absolut error for stiffness matrix interpolation Q_13 is %d, \n', sum(abs((q13-q13Weights)./(eps+q13)))/h^3)
                fprintf('Mean absolut error for stiffness matrix interpolation Q_22 is %d, \n', sum((abs(q22-q22Weights)./(eps+q22)))/h^3)
                fprintf('Mean absolut error for stiffness matrix interpolation Q_23 is %d, \n', sum((abs(q23-q23Weights)./(eps+q23)))/h^3)
                fprintf('Mean absolut error for stiffness matrix interpolation Q_33 is %d, \n', sum((abs(q33-q33Weights)./(eps+q33)))/h^3)
            end
         end
     end
     
end
end


