function [H, opt] = lowRankSVD(H, opt)
%% Singular value decomposition of weight matrices (optional low rank)

if nargin < 2
    opt = struct();
end
if ~isfield(opt, 'lowRank') || isempty(opt.lowRank)
    opt.lowRank = 0;
end
if ~isfield(opt, 'rankTol') || isempty(opt.rankTol)
    if opt.lowRank == 1
        opt.rankTol = 1e-5;
        fprintf('Set truncation tolerance for low rank approximation to %d \n', opt.rankTol);
    else
        opt.rankTol = inf;
    end
end
if ~isfield(opt, 'discardFull') || isempty(opt.discardFull)
    opt.discardFull = 1;
end

fprintf('Size of W is [ %s] \n', sprintf('%d ',H.weightFun.n));
H.stiffness.Rmax = min(H.weightFun.n);
if H.geometry.dim == 2
    % assemble matrices for the decomposition
    if opt.stiffness == 1
        H.stiffness.SVDU = zeros(H.weightFun.n(1), H.stiffness.Rmax, 3);
        H.stiffness.SVDV = zeros(H.weightFun.n(2), H.stiffness.Rmax, 3);
        H.stiffness.SVDWeights = zeros(H.stiffness.Rmax, 3);
        H.stiffness.R = ones(3,1)*H.stiffness.Rmax;
        for i = 1:3
            % svd for each entry
            [U,weights1D,V] = svd(H.stiffness.weightMat(:,:,i));
            weights1D = diag(weights1D);
            if opt.lowRank == 1
                % choose low rank according to truncation tolerance
                for j = 1:numel(weights1D)
                    if weights1D(j) < opt.rankTol
                        H.stiffness.R(i) = j-1;
                        break;
                    end
                end
            else
                % full rank, if no low rank approximation wanted
                H.stiffness.R(i) = H.stiffness.Rmax;
            end

            % truncation of the svd matrices
            H.stiffness.SVDU(:,1:H.stiffness.R(i),i) = U(:,1:H.stiffness.R(i));
            H.stiffness.SVDV(:,1:H.stiffness.R(i),i) = V(:,1:H.stiffness.R(i));
            H.stiffness.SVDWeights(1:H.stiffness.R(i),i) = sqrt(weights1D(1:H.stiffness.R(i)));
        end
        H.stiffness.SVDU = H.stiffness.SVDU(:,1:max(H.stiffness.R),:);
        H.stiffness.SVDV = H.stiffness.SVDV(:,1:max(H.stiffness.R),:);
        H.stiffness.SVDWeights = H.stiffness.SVDWeights(1:max(H.stiffness.R),:);
        fprintf('Low ranks are %i, %i, %i for tolerance %d\n', H.stiffness.R(1), H.stiffness.R(2), H.stiffness.R(3), opt.rankTol);
        if opt.discardFull == 1
            H.stiffness = rmfield(H.stiffness, 'weightMat');
        end
    end
    if opt.mass == 1
        H.mass.Rmax = min(H.weightFun.n);
        H.mass.R = 0;
        [U, weights1D, V] = svd(H.mass.weightMat);
        weights1D = diag(weights1D);
        if opt.lowRank == 1
            H.mass.R = find(weights1D<opt.rankTol,1)-1;
        else
            H.mass.R = H.mass.Rmax;
        end
        H.mass.SVDU = U(:,1:H.mass.R);
        H.mass.SVDV = V(:,1:H.mass.R);
        H.mass.SVDWeights = sqrt(weights1D(1:H.mass.R));
        if opt.discardFull == 1
            H.mass = rmfield(H.mass, 'weightMat');
        end
    end
elseif H.geometry.dim == 3
    if opt.stiffness == 1
        if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
            H.stiffness.SVDU = cell(6,1);
            H.stiffness.R = zeros(6,3);
            for i = 1:6
                if ~isempty(H.stiffness.weightMat{i})
                    H.stiffness.weightMat{i} = round(H.stiffness.weightMat{i}, opt.rankTol);
                    H.stiffness.SVDU{i} = cell(1,3);
                    for j = 1:3
                        H.stiffness.SVDU{i}{j} = zeros(H.stiffness.weightMat{i}.n(j),H.stiffness.weightMat{i}.r(j)*H.stiffness.weightMat{i}.r(j+1));
                        H.stiffness.R(i,j) = H.stiffness.weightMat{i}.r(j)*H.stiffness.weightMat{i}.r(j+1);
                        for k = 1:H.stiffness.weightMat{i}.r(j)
                            for l = 1:H.stiffness.weightMat{i}.r(j+1)
                                H.stiffness.SVDU{i}{j}(:,l+H.stiffness.weightMat{i}.r(j+1)*(k-1)) = H.stiffness.weightMat{i}{j}(k,:,l)';
                            end
                        end
                    end
                else
                    H.stiffness.SVDU{i} = 0;
                    H.stiffness.R(i,:) = 0;
                end
            end                 
        else
            H.stiffness.SVDU = cell(6,1);
            H.stiffness.R = zeros(6,3);
            if strcmp(opt.lowRankMethod,'CPD')
                if isfield(opt,'sizeLowRank') && ~isempty(opt.sizeLowRank)
                    for i = 1:6
                        if ~isempty(H.stiffness.weightMat{i})
                            H.stiffness.SVDU{i} = cpd(H.stiffness.weightMat{i}, opt.sizeLowRank);
                            H.stiffness.R(i,:) = opt.sizeLowRank;  
                        end
                    end
                else
                    fprintf('No low rank sizes given for CPD, changing to mlsvd\n');
                    opt.lowRankMethod = 'MLSVD';
                end
            end
            if strcmp(opt.lowRankMethod, 'MLSVD')
                H.stiffness.SVDWeights = cell(6,1);
                for i = 1:6
                    if ~isempty(H.stiffness.weightMat{i})
                        [H.stiffness.SVDU{i}, H.stiffness.SVDWeights{i}, sv] = mlsvd(H.stiffness.weightMat{i});
                        for k = 1:3
                            R = find(sv{k}>opt.rankTol,1,'last');
                            if isempty(R)
                                H.stiffness.R(i,k) = 1;
                            elseif ~isempty(opt.sizeLowRank)
                                H.stiffness.R(i,k) = opt.sizeLowRank;
                            else
                                H.stiffness.R(i,k) = R;
                            end
                            H.stiffness.SVDU{i}{k} = H.stiffness.SVDU{i}{k}(:,1:H.stiffness.R(i,k));
                        end
                        H.stiffness.SVDWeights{i} = H.stiffness.SVDWeights{i}(1:H.stiffness.R(i,1), 1:H.stiffness.R(i,2), 1:H.stiffness.R(i,3));
                    end
                end
            end
        end
        H.stiffness.order = [1,2,3,2,4,5,3,5,6];
        if opt.discardFull == 1
            H.stiffness = rmfield(H.stiffness, 'weightMat');
        end
        fprintf('Core sizes are [%s] for tolerance %d\n', sprintf('%d ',H.stiffness.R), opt.rankTol)
    end
    if opt.mass == 1
        if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
            H.mass.R = zeros(1,3);
            if ~isempty(H.mass.weightMat)
                H.mass.weightMat = round(H.mass.weightMat, opt.rankTol);
                H.mass.SVDU = cell(1,3);
                for j = 1:3
                    H.mass.SVDU{j} = zeros(H.mass.weightMat.n(j),H.mass.weightMat.r(j)*H.mass.weightMat.r(j+1));
                    H.mass.R(j) = H.mass.weightMat.r(j)*H.mass.weightMat.r(j+1);
                    for k = 1:H.mass.weightMat.r(j)
                        for l = 1:H.mass.weightMat.r(j+1)
                            H.mass.SVDU{j}(:,l+H.mass.weightMat.r(j+1)*(k-1)) = H.mass.weightMat{j}(k,:,l)';
                        end
                    end
                end
            else
                H.mass.SVDU = 0;
                H.mass.R(:) = 0;
            end                
        else
            H.mass.R = zeros(3,1);
            if strcmp(opt.lowRankMethod,'CPD') 
                if isfield(opt, 'sizeLowRank') && ~isempty(opt.sizeLowRank)
                    H.mass.SVDU = cpd(H.mass.weightMat, opt.sizeLowRank);
                    H.mass.R(:) = opt.sizeLowRank;
                else
                    fprintf('No low rank sizes given for CPD, changing to mlsvd\n');
                    opt.lowRankMethod = 'MLSVD';
                end
            end
            if strcmp(opt.lowRankMethod, 'MLSVD')
                [H.mass.SVDU, H.mass.SVDWeights, sv] = mlsvd(H.mass.weightMat);
                for i = 1:3
                    if ~isempty(opt.sizeLowRank)
                        H.mass.R(i) = opt.sizeLowRank;
                    else
                        H.mass.R(i) = find(sv{i}>opt.rankTol, 1, 'last');
                        if isempty(H.mass.R(i))
                            H.mass.R(i) = 1;
                        end
                    end
                    H.mass.SVDU{i} = H.mass.SVDU{i}(:,1:H.mass.R(i));
                end
                H.mass.SVDWeights = H.mass.SVDWeights(1:H.mass.R(1), 1:H.mass.R(2), 1:H.mass.R(3));
            end
        end
        if opt.discardFull == 1
            H.mass = rmfield(H.mass, 'weightMat');
        end
    end
end
    

end

