function G = readGeo(file)

%% prepares struct from geometry structures from "ISOGAT" or "geoPDEs" or generation .tex or .mat files (structure as in geoPDEs files) for further processing with Low Rank IGA codes
% input: geometry file from "ISOGAT" or "geoPDES" either preloaded or
% generation file
% return: struct G with fields
%   nurbs:  1 if NURBS structure or 0 if B-splines
%   dim:    geometry dimension
%   knots:  cell with knot vectors for each dimension
%   degree: array with dimension of splines in each dimension
%   n:      array with size of spline space in each dimension
%   weight: tensor with weights for each d-dimensional spline, size n
%   controlPoints:  tensor with control points for each d-dimensional
%           spline, size n,d
if isstruct(file) && isfield(file,'weight')
    G = file;
    return;
end
G = struct;

% case 1: filename is an isogat class
if isa(file, 'isogat') == 1
    G.nurbs = 1;
    G.dim = 2;
    G.knots{1} = file.knotVectorU;
    G.knots{2} = file.knotVectorV;
    G.degree(1) = file.pU;
    G.degree(2) = file.pV;
    G.n = [numel(G.knots{1}) - G.degree(1) - 1, numel(G.knots{2}) - G.degree(2) - 1];
    G.weight = reshape(file.weights, G.n(1), G.n(2));
    G.controlPoints = reshape(file.controlPoints, G.n(1), G.n(2),2);
else
    % case 2: filename is a preloaded geopdes geometry structure
    if isa(file, 'struct') == 1
        H = file;
    else
    % case 3: filename is a geopdes geometry file (.txt or .mat)
        H = geo_load(file);
    end
    if isfield(H, 'geometry')
        % some geopedes store geometry in substructure
         H = H.geometry;
    end
    if isfield(H, 'nurbs')
        % another substructure if the splines are NURBS
        H = H.nurbs;
        G.nurbs = 1;
    else
        G.nurbs = 0;
    end
    % 2D
    if numel(H.order) == 2
        G.dim = 2;
        G.knots{1} = H.knots{1};
        G.knots{2} = H.knots{2};
        G.degree(1) = H.order(1) - 1;
        G.degree(2) = H.order(2) - 1;
        G.n = H.number;
        G.weight = reshape(H.coefs(4,:,:),G.n);
        G.controlPoints = zeros(G.n(1),G.n(2),2);
        G.controlPoints(:,:,1) = reshape(H.coefs(1,:,:)./H.coefs(4,:,:),G.n);
        G.controlPoints(:,:,2) = reshape(H.coefs(2,:,:)./H.coefs(4,:,:),G.n);
    % 3D
    elseif numel(H.order) == 3
        G.dim = 3;
        G.knots{1} = H.knots{1};
        G.knots{2} = H.knots{2};
        G.knots{3} = H.knots{3};
        G.degree = H.order - 1;
        G.n = H.number;
        G.weight = reshape(H.coefs(4,:,:,:),G.n);
        G.controlPoints = zeros(G.n(1),G.n(2),G.n(3),3);
        G.controlPoints(:,:,:,1) = reshape(H.coefs(1,:,:,:)./H.coefs(4,:,:,:), G.n);
        G.controlPoints(:,:,:,2) = reshape(H.coefs(2,:,:,:)./H.coefs(4,:,:,:), G.n);
        G.controlPoints(:,:,:,3) = reshape(H.coefs(3,:,:,:)./H.coefs(4,:,:,:), G.n);
    end
end
for i = 1:G.dim
    if G.knots{i}(end) > 1
        % knot vectors should have domain [0,1]
        fprintf('note: changed knot vector in dimension %i from [0,%i] to [0,1]\n', i, G.knots{i}(end));
        G.knots{i} = G.knots{i}/G.knots{i}(end); 
    end
end
end
    
    
    