function [solTT,max_res,swp, max_rank] = optimalControl_TT(H, yhat, param, My0)

% input:
%   - geometry data structure H in TT format
%   - desired state vector yhat
%   - (optional) optimization parameter struct param with fields
%       - beta: coparam.ntrol parameter
%       - nt: number of time steps
%       - T: end time
%       - tol: convergence tolerance
% output:
%   solution solTT where
%       - solTT(1,:) solution for state y
%       - solTT(2,:) solution for control u
%       - solTT(3,:) solution for Lagrange multipliers lambda
%   

if nargin < 3
    param = struct;
    param.beta = 1e-2;
    param.nt = 10;
    param.T = 1;
    param.tol = 1e-4;
    param.preconditioner = 0;
end

dt = param.T/param.nt;
D = speye(param.nt,param.nt);
e = ones(param.nt,1);
C = spdiags([-e,e],-1:0,param.nt,param.nt);
%% Add all the terms from the  low-rank IGA in TT format
TT_M = cell(4, H.mass.R(2));
for i=1:H.mass.R(1)
    for j = 1:H.mass.R(3)
        % Set up mass matrix ( kron(I,M) )
        TT_M{1,j+(i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.X{i});
        TT_M{2,j+(i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.Y{i+(j-1)*H.mass.R(1)});
        TT_M{3,j+(i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.Z{j});
        TT_M{4,j+(i-1)*H.mass.R(3)} = D;
    end
end

TT_K = cell(4,sum(H.stiffness.R(H.stiffness.order,2))+H.mass.R(2));
for i=1:9 % set up stiffness matrix ( kron(C,M)+kron(I,dt*K) )
    for j = 1:H.stiffness.R(H.stiffness.order(i),1)
        for k = 1:H.stiffness.R(H.stiffness.order(i),3)
            TT_K{1,sum(H.stiffness.R(H.stiffness.order(1:(i-1)),2))+(k+(j-1)*H.stiffness.R(H.stiffness.order(i),3))} = dt^(1/3)*sparse(H.stiffness.X{i}{j});
            TT_K{2,sum(H.stiffness.R(H.stiffness.order(1:(i-1)),2))+(k+(j-1)*H.stiffness.R(H.stiffness.order(i),3))} = dt^(1/3)*sparse(H.stiffness.Y{i}{k+(j-1)*H.stiffness.R(H.stiffness.order(i),3)});
            TT_K{3,sum(H.stiffness.R(H.stiffness.order(1:(i-1)),2))+(k+(j-1)*H.stiffness.R(H.stiffness.order(i),3))} = dt^(1/3)*sparse(H.stiffness.Z{i}{k});
            TT_K{4,sum(H.stiffness.R(H.stiffness.order(1:(i-1)),2))+(k+(j-1)*H.stiffness.R(H.stiffness.order(i),3))} = D;
       end
    end
end
for i = 1:H.mass.R(1)
    for j = 1:H.mass.R(3)
        TT_K{1,sum(H.stiffness.R(H.stiffness.order,2))+j+(i-1)*H.mass.R(3)} = sparse(H.mass.X{i});
        TT_K{2,sum(H.stiffness.R(H.stiffness.order,2))+j+(i-1)*H.mass.R(3)} = sparse(H.mass.Y{i+(j-1)*H.mass.R(1)});
        TT_K{3,sum(H.stiffness.R(H.stiffness.order,2))+j+(i-1)*H.mass.R(3)} = sparse(H.mass.Z{j});
        TT_K{4,sum(H.stiffness.R(H.stiffness.order,2))+j+(i-1)*H.mass.R(3)} = C;
     end
end

TT_betaM = TT_M;
TT_minusM = TT_M;
for i = 1:size(TT_M,2)
    TT_betaM{4,i} = TT_betaM{4,i}*param.beta;
    TT_minusM{4,i} = -TT_minusM{4,i};
end

% KKT Matrix [M, 0, K'; 0, beta*M, -M; K, -M, 0]
% with M = kron(I,dt*M), K = kron(I,dt*K)+kron(C,M)
TT_all = {TT_M, [], cellfun(@(x) x', TT_K, 'uniformOutput',0); [], TT_betaM, TT_minusM; TT_K, TT_minusM, []};

% creat right hand side in TT-format [M*yhat; 0; 0]
% with yhat constant in time
MM = cell(3,H.mass.R(2));
for i = 1:H.mass.R(1)
    for j = 1:H.mass.R(3)
        MM{1,j + (i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.X{i});
        MM{2,j + (i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.Y{i+(j-1)*H.mass.R(1)});
        MM{3,j + (i-1)*H.mass.R(3)} = dt^(1/3)*sparse(H.mass.Z{j});
    end   
end
if numel(yhat) == prod(H.geometry.n-2)
    if ~isfield(H,'massMat')
        My = tt_matrix(MM)*yhat;
        rhs = tt_tensor(reshape(kron(ones(param.nt,1),My),[size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt]));
    else
        rhs = tt_tensor(reshape(kron(ones(param.nt,1),dt*H.massMat*yhat),[size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt]));
    end
elseif numel(yhat) == prod(H.geometry.n-2)*param.nt
    if ~isfield(H,'massMat')
        My = tt_matrix(MM)*reshape(yhat,prod(H.geometry.n-2),param.nt);
        rhs = tt_tensor(reshape(My,[size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1), param.nt]));
    else
        rhs = tt_tensor(reshape(dt*H.massMat*yhat,[size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt]));
    end
else
    error('dimensionality of yhat is not suitable');
end


TT_rhs = {rhs; tt_tensor(zeros([size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt])); tt_tensor(zeros([size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt]))};

if nargin == 4
    y0 = tt_tensor(reshape([My0,zeros(numel(My0),param.nt-1)],[size(H.mass.X{1},1),size(H.mass.Y{1},1),size(H.mass.Z{1},1),param.nt]));
    TT_rhs{3} = y0;
end

% solve with amen_block_solve
if isfield(param,'preconditioner') && param.preconditioner == 1
    [solTT,max_res,swp, max_rank] = amen_block_solve(TT_all, TT_rhs, param.tol, 'kickrank', 2, 'resid_damp', 1e2, 'exitdir', -1,...
        'solfun', @(i, XAX1,Ai,XAX2, XY1,yi,XY2, tol, sol_prev)locsol2(i, XAX1,Ai,XAX2, XY1,yi,XY2, 1e-6, sol_prev));
else
    [solTT,max_res,swp, max_rank] = amen_block_solve(TT_all, TT_rhs, param.tol, 'kickrank', 2, 'resid_damp', 1e2, 'exitdir', -1);
end
% solTT = full(XX_sol);

% %% plot solution at last timestep k = nt
% k = param.nt;    'nswp', 3,
% p = figure;
% hold on
% subplot(1,2,1)
% p = plotNSol(H, solY((k-1)*size(solY,2)/param.nt+1:k*size(solY,2)/param.nt)',p);
% title(['Solution at timestep ' num2str(k)])
% subplot(1,2,2)
% plotNSol(H,yhat,p);
% title('constant yhat')

%% plot solution at last timestep k = nt
% k = param.nt;
% p = figure;
% hold on
% subplot(1,2,1)
% p = plotNSol(H, solY((k-1)*size(solY,2)/param.nt+1:k*size(solY,2)/param.nt)',p);
% title(['Solution at timestep ' num2str(k)])
% subplot(1,2,2)
% if numel(yhat) == prod(H.geometry.n-2)
%     plotNSol(H,yhat,p);
% else
%     plotNSol(H,yhat((k-1)*prod(H.geometry.n-2)+1:k*prod(H.geometry.n-2)),p);
% end
% title('constant yhat') 
end

