function [H,opt] = system_generation(filename, opt)

%% generic system generation function from any geometry file 'filename' with options 'opt' as explained more detailed in the example2D.m file
% input: 
% - geometry file 'filename'
% - opt can hold specifications with the following 1/0 options [default]
%       plotW:        plots grid of weight approximation accuracy [0]
%       lowRank:      reduce rank for the SVD [1]
%       mass:         1 if mass matrix should be assembled [1]
%       stiffness:    1 if stiffness matrix should be assembled [1]
%       discardFull:  discards the assembled weight matrices [1]
%       and the following options, which require pre-estimated values if chosen,
%       else leave blank:
%       sizeLowRank:  fixed size for the SVD appriximation as integer, []
%       rankTol:      tolerance for rank truncation [default 1e-5]
%       refinement:   number of refinement steps for geometry as integer []
%       elevation:    degree elevation of geometry representation as d-dimensional array of integers []
%       lowRankMethod:method of low rank approximation, 'MLSVD' or 'CPD' ['CPD']
%
% output: 
% Structure H which holds:
%       H.geometry - geometry information
%       H.weightFun - information about the interpolation space
%       H.mass  - information about the mass matrix
%               H.mass.weightMat - the full weight interpolation tensor (only if opt.discard = 0)
%               H.mass.R - the reduced ranks of the weight interpolation
%               H.mass.SVDU - a cell array with TT-decomposition of the weight tensor
%               H.mass.X - the 1D mass matrices which assemble the whole mass matrix as a sum of kroenecker products
%               H.mass.Y - the second 1D mass matrices
%               H.mass.Z - the third dimensiton of the 1D mass matrices
%       H.stiffness - information about the stiffness matrix in the same structure as H.mass, except that the interpolated weight function has 9 components
%               H.stiffness.SVDU - a cell array with TT-decomposition of the 6 interpolated weight tensors
%               H.stiffness.Rmax - the max rank of the tensors
%               H.stiffness.R - the reduced TT-ranks for all 6 components   "
%               H.stiffness.X, .Y, .Z - the 1D stiffness matrices
%       H.boundary - boundary information (just 0 dirichlet until now)
%       H.xInner, H.yInnter,H.zInner - non boundary splines in each direction
% only if opt.discardFull == 0 :
%       H.massMat - mass matrix assembled from low rank format
%       H.stiffnessMat - stiffness matrix assembled from low rank format


if nargin < 2 || ~isstruct(opt)
    opt = struct;
    opt.plotW = 0;
    opt.lowRank = 1;
    opt.mass = 1;
    opt.stiffness = 1;
    opt.discardFull = 1;
    opt.sizeLowRank = [];
    opt.rankTol = 1e-5;
    opt.refinement = [];
    opt.elevation = [];
    opt.lowRankMethod = 'CPD';
    opt.TT_interpolation = 1;
end

% load right geometry format
geometry = readGeo(filename);

if isfield(opt, 'refinement') && ~isempty(opt.refinement) && opt.refinement ~=0
    % optional degree elevation and knot refinement
    if isfield(opt,'elevation') && ~isempty(opt.elevation)
        geometry = refinement(geometry, opt.refinement, opt.elevation);
    else
        geometry = refinement(geometry, opt.refinement);
    end
end

% format NURBS
geometry = fitNURBS(geometry,0);
t = tic;
% assemble system matrices
[H,opt] = assemble(geometry,opt);
t = toc(t);
if isfield(H,'stiffnessMat') || isfield(H, 'massMat')
    matFormat = 'including';
else
    matFormat = 'excluding';
end
fprintf('Elapsed time for system matrix generation is %d s with %i degrees of freedom (%s full matrix assembly). \n',t, prod(H.geometry.n-2), matFormat);
end